/**************************************************
opengate mac addr auth program

 cgi module for reg/update by owner oneself

Copyright (C) 2011 Opengate Project Team
Written by Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include	"opengatemmng.h"

/***************************************************/
/*  main routine called as cgi from Web server     */
/***************************************************/
int  main(int argc, char **argv)
{
  char language[WORDMAXLN]="";  /* browser prefered language(e.g.:ja,en) */
  char userId[USERMAXLN]="";    /* user id */
  char extraId[USERMAXLN]="";   /* extra id used as user@extra */
  char requestStr[BUFFMAXLN]=""; /* http request string */
  char* progName="";             /* the name of this program in argv[0] */
  char mailDefault[BUFFMAXLN]=""; /* default mail address to get warning */
  char macAddress[ADDRMAXLN]="";  /* client MAC address */
  char macAddressReq[ADDRMAXLN]=""; /* MAC address in request text */
  char ipAddress[ADDRMAXLN]="";     /* IP address */
  char* dev;                        /* name of network interface */
  char ipAddressInQueue[ADDRMAXLN]="";  /* IP address found in queue */
  char macAddressInQueue[ADDRMAXLN]=""; /* MAC address found in queue */ 
  char deviceName[WORDMAXLN]=""; /* client device name */
  char mailAddress[BUFFMAXLN]="";  /* user's mail address to get warning */
  int isNatOrRouter=0; /* flag for nat or router insertion (checked in md) */
  char redirectedUrl[BUFFMAXLN]=""; /* url from which the page is forwarded */
  int modified=FALSE; /* register or update is done */

  /* drop root privilege (to prevent dangerous action */
  seteuid(getuid());

  /* if this is executed in shell with '-v' option, show makedir */
  /* this is prepared to show information about version */
  if(argc>1){
    if(strcmp(argv[1],"-v")==0){
      printf("makedir: %s\n", MAKEDIR);
    }else{
      printf("This is cgi program\n");
      printf("To show version, run this on console with '-v' option\n");
    }
    exit(0);
  }

  /* save program load path */
  saveLoadPath(argv[0]);
  progName = getProgramName();

  /* prepare configuration file */
  if(OpenConfFile()==-1){
    PutMessageToClient("Check config file by running this cgi on console");
    return 0;
  }
 
  /* start log */
  errToSyslog(atoi(GetConfValue("Syslog/Enable")));
  openlog(progName, LOG_PID, atoi(GetConfValue("Syslog/Facility")));

  /* initialize configuration and database */
  InitConf();
  if(!InitMngDb()) return 0;
  if(!InitWorkDb()) return 0;

  /* get language and other from query string */
  GetLangFromQueryString(language);
  GetRedirectedUrlFromQueryString(redirectedUrl);

  /* check client's http-agent, if the agent is not allowed in conf, exit */
  if(!IsAccessedFromAllowableAgent()){
    SetMessage(UnallowableAgent);
    PutDenyToClient(language);
    CloseMngDb();
    return 0;
  }

  /*** Get request from client ***/
  /* get post data */
  GetPostData(requestStr, BUFFMAXLN);

  /* get cgi name defined in conf file */
  if(isNull(GetConfValue("OwnCgi"))){
    err_msg("ERR at %s#%d: OwnCgi cannot get from conf file",__FILE__,__LINE__);
    return 0;
  }

  /* get userid and other info. */
  /* if not get, send auth page and exit */
  if(!GetUserId(requestStr, userId, extraId, language, NORMALUSER,
		GetConfValue("OwnCgi"),mailDefault,redirectedUrl)){
    CloseMngDb();
    return 0;
  }

  /* if too many requests per one day, send error and exit */
  if(CountMacModifyPerDayInMngDb(userId,extraId) 
     > atoi(GetConfValue("MaxMacModifyPerDay"))){
    SetMessage(ModifyCountOver);
    PutDenyToClient(language);
    CloseMngDb();
    return 0;
  }

  /*** Get client MAC address ***/
  /* get the client ip address from environment variable */
  strncpy(ipAddress, getenv("REMOTE_ADDR"), ADDRMAXLN);

  /* get nic interface name in conf */
  if(isNull(dev=GetConfValue("Device"))){
        err_msg("ERR at %s#%d: Device cannot get from conf file",__FILE__,__LINE__);
	return 0;
  }

  /* get IP-MAC pair list from arp or ndp on the nic */
  Initqueue();
  if(strchr(ipAddress,':')==NULL) GetMacAddrListFromArp(dev);
  else GetMacAddrListFromNdp(dev);

  /* search mac address corresponding to the client ip address */
  while(Dequeue(macAddressInQueue, ipAddressInQueue)){
    if(strcmp(ipAddress, ipAddressInQueue)==0){
      strncpy(macAddress, macAddressInQueue, ADDRMAXLN);
      break;
    }
  }

  /* clear the ip-mac pair list */
  Freequeue();

  /*** Check the acquired MAC ***/
  /* if accessed via nat or router, exit  */
  /* if nat, check function returns 1. if router, it returns 2. */
  isNatOrRouter=IsNatSuspectedInWorkDb(macAddress);
  if(isNatOrRouter==NAT || isNatOrRouter==ROUTER){
    SetMessage(SuspectNat);
    PutDenyToClient(language);
    CloseMngDb();
    return 0;
  }

  /*** execute the client request ***/
  /* if null request, skip execution and go to sending page */
  if(!isNull(requestStr)){
    
    /* execute update requests */
    if(AnalyzeUpdateRequestAndExecute(requestStr, userId, extraId)){
      SetMessage(UpdateSuccess);
      modified=TRUE;
    }

    /* if no update, check registration request */
    else if(AnalyzeRegisterRequest(requestStr, macAddressReq, 
			      deviceName, mailAddress)){
      
      /* if registration info is found, register values to db */
      RegistMacAddrToMngDb(macAddress,deviceName,userId,extraId,mailAddress);
      PutMacModifyLogToMngDb(userId, extraId, macAddress, 'R');
      PutMacAddressToServers(macAddress);
      SetMessage(RegisterSuccess);
      modified=TRUE;
    }
  }

  /*** Send back web page ***/
  /* if registration/Updating is done, return to redirected site */
  if(modified && !isNull(redirectedUrl)){
    ReturnToRedirectedPage(redirectedUrl, language);
  }

  /* else if the client is registered, send back update page */
  else if(IsMacAddrFoundInMngDb(macAddress)){
    PutUpdatePageToClient(language, userId, extraId, OWNER, redirectedUrl);
  }

  /* client is not-registered */
  /* but allowable registration device count is over the limit */
  /* send back update page with overflow error message */
  else if(CountMacAddrsForUserInMngDb(userId,extraId) 
	  >= atoi(GetConfValue("MaxDevices"))){
    SetMessage(DeviceCountOver);
    PutUpdatePageToClient(language, userId, extraId, OWNER, redirectedUrl);
  }

  /* client is not-registered and device count is under the limit */
  else{

    /* restore mail address */
    if(isNull(mailAddress)) strncpy(mailAddress, mailDefault, BUFFMAXLN);

    /* send registration page to the client */
    PutRegisterPageToClient(language, macAddress, deviceName, mailAddress, 
			    userId, extraId, OWNER, redirectedUrl);
  }

  /* finalize */
  CloseMngDb();
  return 0;
}

